@@ -6,7 +6,7 @@
 ;; Maintainer: Nicolas Goaziou <mail@nicolasgoaziou.fr>
 ;; Keywords: web
 ;; Package-Requires: ((emacs "26.1"))
-;; Version: 1.1.0
+;; Version: 1.2.1
 
 ;; This program is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -65,6 +65,30 @@
 ;;                 (repology-search-projects
 ;;                  :search "emacs" :inrepo "gnuguix" :outdated "on")))
 
+;; By default, the package trusts Repology's status values to report
+;; outdated packages.  However, this can introduce false positives.
+;; You can then set `repology-outdated-project-definition' and call
+;; `repology-filter-outdated-projects' to ignore those.
+
+;; For example, with the following set-up, I can look for every
+;; outdated Emacs packages and Asymptote package in GNU Guix, ignoring
+;; bogus versions for "emacs:circe", and "emacs:erlang" package
+;; altogether.  I also sort projects alphabetically.
+
+;;    (setq repology-outdated-project-definition
+;;          '(("emacs:circe" "<=2.11" nil)
+;;            ("emacs:erlang" nil nil))
+;;          repology-display-projects-sort-key "Project")
+;;
+;;    (let ((repo "gnuguix"))
+;;      (repology-display-projects
+;;       (repology-filter-outdated-projects
+;;           (append (repology-search-projects :search "emacs:" :outdated "on"
+;;                                             :inrepo repo)
+;;                   '("asymptote"))
+;;           repo)
+;;       repo))
+
 ;; Eventually, this library provides an interactive function with
 ;; a spartan interface wrapping this up: `repology'.  Since it builds
 ;; and displays incrementally search filters, you may use it as
@@ -138,6 +162,55 @@ See `repology-check-freedom' for more information."
           (const :tag "Free and unknown projects" include-unknown)
           (const :tag "Every project" nil)))
 
+(defcustom repology-outdated-project-definition nil
+  "Determine how projects are considered as \"outdated\".
+
+This function affects `repology-filter-outdated-projects' function.
+
+When nil, a project is considered as \"outdated\" relatively to
+a repository whenever the corresponding package from that
+repository has the \"outdated\" status.
+
+Otherwise, it can be set to a list of masks.  A mask is a triplet
+
+  (NAME VERSION REPOSITORY)
+
+where NAME is a regexp or nil, VERSION is a string prefixed with
+either \"<\", \"<=\", \"=\", \">\" or \">=\", and REPOSITORY is
+a regexp or nil.
+
+Project's packages are matched against every non-nil criteria in
+the mask.  For example, the mask
+
+  (\"^Foo$\" \"=2\" nil)
+
+matches against project Foo at version 2 (or 2.0.0) only, in any
+repository.
+
+The mask
+
+  (nil nil \"BSD\")
+
+matches against any project from *BSD repositories.
+
+In this case, a project is \"outdated\" when the version of the
+package from the reference repository is older than the version
+of any non-masked package."
+  :type
+  '(choice
+    (const :tag "Use Repology packages status" nil)
+    (repeat :tag "Compare version with unmasked packages"
+     (list :tag "Mask"
+           (choice
+            (regexp :tag "Regexp matching project name")
+            (const :tag "Any project" nil))
+           (choice
+            (string :tag "Version comparison string")
+            (const :tag "Any version" nil))
+           (choice
+            (regexp :tag "Regexp matching repository name")
+            (const :tag "Any repository" nil))))))
+
 (defcustom repology-display-problems-columns
   `(("Project" effname 20 t)
     ("Package name" visiblename 20 t)
@@ -162,6 +235,28 @@ problem argument.  Its return value is then turned into a string and displayed."
                   (const :tag "Sort" t)
                   (function :tag "Custom sort predicate")))))
 
+(defcustom repology-display-problems-sort-key nil
+  "Initial sort key used when displaying a list of problems.
+
+If nil, no additional sorting is performed.
+
+Otherwise, this should be a cons cell (NAME . FLIP).  NAME is
+a string matching one of the column names in
+`repology-display-problems-columns'.  FLIP, if non-nil, means to
+invert the resulting sort.
+
+Alternatively, it can be a string matching a valid column name.
+In that case, FLIP is considered as nil.
+
+If the key name doesn't match any column name, no sorting is
+initially done."
+  :type
+  '(choice
+    (const :tag "No sorting" nil)
+    (cons :tag "Sort key"
+          (string :tag "Column name")
+          (boolean :tag "Reversed?"))))
+
 (defcustom repology-display-packages-columns
   '(("Repository"
      repology-package-repository-full-name
@@ -202,6 +297,28 @@ You may also want to look into comparison functions suitable for SORT, such as
                   (const :tag "Sort" t)
                   (function :tag "Custom sort predicate")))))
 
+(defcustom repology-display-packages-sort-key nil
+  "Initial sort key used when displaying a list of packages.
+
+If nil, no additional sorting is performed.
+
+Otherwise, this should be a cons cell (NAME . FLIP).  NAME is
+a string matching one of the column names in
+`repology-display-packages-columns'.  FLIP, if non-nil, means to
+invert the resulting sort.
+
+Alternatively, it can be a string matching a valid column name.
+In that case, FLIP is considered as nil.
+
+If the key name doesn't match any column name, no sorting is
+initially done."
+  :type
+  '(choice
+    (const :tag "No sorting" nil)
+    (cons :tag "Sort key"
+          (string :tag "Column name")
+          (boolean :tag "Reversed?"))))
+
 (defcustom repology-display-projects-columns #'repology-display-projects-default
   "Columns format rules used to display a list of projects.
 
@@ -234,6 +351,28 @@ You may also want to look into comparison functions suitable for SORT, such as
                          (function :tag "Custom sort predicate"))))
           (function :tag "Function describing columns")))
 
+(defcustom repology-display-projects-sort-key nil
+  "Initial sort key used when displaying a list of projects.
+
+If nil, no additional sorting is performed.
+
+Otherwise, this should be a cons cell (NAME . FLIP).  NAME is
+a string matching one of the column names in
+`repology-display-projects-columns'.  FLIP, if non-nil, means to
+invert the resulting sort.
+
+Alternatively, it can be a string matching a valid column name.
+In that case, FLIP is considered as nil.
+
+If the key name doesn't match any column name, no sorting is
+initially done."
+  :type
+  '(choice
+    (const :tag "No sorting" nil)
+    (cons :tag "Sort key"
+          (string :tag "Column name")
+          (boolean :tag "Reversed?"))))
+
 
 ;;; Faces
 (defgroup repology-faces nil
@@ -515,6 +654,35 @@ according to the value of `repology-free-projects-only'."
                       (lambda (p) (eq t (repology-check-freedom p))))
                     result)))))
 
+;;;###autoload
+(defun repology-collect-projects (names)
+  "Collect a list of Repology projects by their name.
+
+NAMES is a list of project names, as strings.
+
+Project names can also be a list of strings.  In that case, the
+project is named after the first element of the list and packages
+associated to subsequent names are merged into it, as if all were
+a single project.  This is useful when Repology has multiple names
+from the same project, or when you want the project to be
+displayed under a different name than Repology's.
+
+Return a list of Repology projects, in the order specified in
+NAMES."
+  (mapcar (lambda (name)
+            (let* ((project
+                    (pcase name
+                      (`(,project . ,_) project)
+                      (_ name)))
+                   (packages
+                    (cond
+                     ((listp name)
+                      (seq-mapcat #'repology-lookup-project name))
+                     (t
+                      (repology-lookup-project name)))))
+              (repology-project-create project packages)))
+          names))
+
 ;;;###autoload
 (defun repology-report-problems (repository)
   "List problems related to REPOSITORY.
@@ -564,6 +732,22 @@ REPOSITORY is a string.  Return a list of problems."
   (repology-display-packages
    (repology-project-packages (tabulated-list-get-id))))
 
+(defun repology--display-sort-key (key-pair)
+  "Return initial sort key for current list.
+
+KEY-PAIR is the sort key to consider.  Return nil if KEY-PAIR is
+nil, or if the name of the column to sort initially does not
+exist.
+
+This function assumes `tabulated-list-format' is set already."
+  (pcase key-pair
+    ((or (and (pred stringp) name) `(,name . ,flip))
+     (let ((columns (mapcar #'car tabulated-list-format))
+           (flip (bound-and-true-p flip))) ;not defined when string
+       (and (member name columns)
+            (cons name flip))))
+    (_ nil)))
+
 (define-derived-mode repology--display-package-mode tabulated-list-mode
   "Repology/Package"
   "Major mode used to display packages returned by Repology API.
@@ -577,6 +761,8 @@ REPOSITORY is a string.  Return a list of problems."
 \\{repology--display-packages-mode-map}"
   (setq tabulated-list-format
         (repology--columns-to-header repology-display-packages-columns))
+  (setq tabulated-list-sort-key
+        (repology--display-sort-key repology-display-packages-sort-key))
   (tabulated-list-init-header))
 
 (define-derived-mode repology--display-projects-mode tabulated-list-mode
@@ -585,6 +771,8 @@ REPOSITORY is a string.  Return a list of problems."
 \\{repology--display-projects-mode-map}"
   (setq tabulated-list-format
         (repology--columns-to-header repology-display-projects-columns))
+  (setq tabulated-list-sort-key
+        (repology--display-sort-key repology-display-projects-sort-key))
   (tabulated-list-init-header))
 
 (define-derived-mode repology--display-problems-mode tabulated-list-mode
@@ -593,6 +781,8 @@ REPOSITORY is a string.  Return a list of problems."
 \\{tabulated-list-mode-map}"
   (setq tabulated-list-format
         (repology--columns-to-header repology-display-problems-columns))
+  (setq tabulated-list-sort-key
+        (repology--display-sort-key repology-display-problems-sort-key))
   (tabulated-list-init-header))
 
 (defun repology--value-to-string (value)
